
const St = imports.gi.St;
const Main = imports.ui.main;
const Clutter = imports.gi.Clutter;
const Meta = imports.gi.Meta;

let panel,
    panelBinding,
    lastScroll,
    scrollDelay,
    button,
    buttonLabel,
    workspaceSwitchedId;

function init() {
  panel = Main.panel;
  panelBinding = null;
  lastScroll = Date.now();
  scrollDelay = 200; // TODO: Take this from pref.

  button = new St.Bin({ style_class: 'panel-button',
                        reactive: true,
                        can_focus: true,
                        x_fill: true,
                        y_fill: false,
                        track_hover: true });
}

function enable() {
  panel.reactive = true;
  if (panelBinding) {
    disable();
  }

  button.connect('button-press-event', _move);
  _updateLabel();

  panelBinding = panel.actor.connect('scroll-event',_onScroll);
  Main.panel._rightBox.insert_child_at_index(button, 0);
  workspaceSwitchedId = global.screen.connect('workspace-switched', _updateLabel);
}

function disable() {
  if (panelBinding) {
    panel.actor.disconnect(panelBinding);
    panelBinding = null;
  }
  Main.panel._rightBox.remove_child(button);
  global.screen.disconnect(workspaceSwitchedId);
}

function _onScroll(actor, event) {
  let source = event.get_source();
  if (source != actor && source != button) {
    let inStatusArea = panel._rightBox && panel._rightBox.contains && panel._rightBox.contain(source);
    if (inStatusArea) {
      return Clutter.EVENT_PROPAGATE;
    }
  }

  let motion;
  switch(event.get_scroll_direction()) {
    case Clutter.ScrollDirection.UP:
      motion = Meta.MotionDirection.UP;
      break;
    case Clutter.ScrollDirection.DOWN:
      motion = Meta.MotionDirection.DOWN;
      break;
    case Clutter.ScrollDirection.LEFT:
      motion = Meta.MotionDirection.LEFT;
      break;
    case Clutter.ScrollDirection.RIGHT:
      motion = Meta.MotionDirection.RIGHT;
      break;
    default:
      return Clutter.EVENT_PROPAGATE;
  }
  let activeWs = global.screen.get_active_workspace();
  let ws = activeWs.get_neighbor(motion);
  if(!ws) return Clutter.EVENT_STOP;

  let currentTime = Date.now();
  if (currentTime < lastScroll + scrollDelay) {
    if(currentTime<lastScroll) {
      lastScroll = 0;
    }
    else {
      return Clutter.EVENT_STOP;
    }
  }

  lastScroll = currentTime;
  Main.wm.actionMoveWorkspace(ws);
  return Clutter.EVENT_STOP;
}

function _move(actor, event) {
  let activeWsId = global.screen.get_active_workspace().index();
  let totalWs = global.screen.n_workspaces;

  if (activeWsId == totalWs-1) {
    let newWs = global.screen.get_workspace_by_index(0);
    newWs.activate(global.get_current_time());
    return;
  }

  let activeWs = global.screen.get_active_workspace();
  let ws = activeWs.get_neighbor(Meta.MotionDirection.DOWN);
  Main.wm.actionMoveWorkspace(ws);
}

function _updateLabel() {
  let activeWsId = global.screen.get_active_workspace().index() + 1;
  buttonLabel = new St.Label({ text: ' '+ activeWsId +' ' });
  button.set_child(buttonLabel);
}